<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\InlineMod\Post;

use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

/**
 * An inline moderation action for converting posts to conversations.
 */
class ConvertToConversation extends AbstractAction
{
    /**
     * @return \XF\Phrase
     */
    public function getTitle()
    {
        return \XF::phrase('sv_modess_convert_posts_to_conversation...');
    }

    /**
     * @param Entity $entity
     * @param array  $options
     * @param string $error
     *
     * @return bool
     */
    protected function canApplyToEntity(
        Entity $entity,
        array $options,
        &$error = null
    ) {
        /** @var \SV\ModeratorEssentials\XF\Entity\Post $entity */
        return $entity->canConvertToConversation($error);
    }

    /**
     * @param AbstractCollection $entities
     * @param array              $options
     */
    protected function applyInternal(
        AbstractCollection $entities,
        array $options
    ) {
        /** @var \SV\ModeratorEssentials\Service\Post\ConversationConverter $converter */
        $converter = $this->app()->service(
            'SV\ModeratorEssentials\Service\Post\ConversationConverter',
            $entities,
            $options
        );
        $conversation = $converter->convert();

        $visitor = \XF::visitor();
        $recipients = $converter->getRecipients();
        if (isset($recipients[$visitor->user_id])) {
            $this->returnUrl = $this->app()->router('public')->buildLink(
                'conversations',
                $conversation
            );
        }
    }

    /**
     * @param Entity $entity
     * @param array  $options
     */
    protected function applyToEntity(Entity $entity, array $options)
    {
    }

    /**
     * @return array
     */
    public function getBaseOptions()
    {
        return [
            'type'    => 'soft',
            'reason'  => '',
            'message' => ''
        ];
    }

    /**
     * @param AbstractCollection $entities
     * @param \XF\Mvc\Controller $controller
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function renderForm(
        AbstractCollection $entities,
        \XF\Mvc\Controller $controller
    ) {
        $posts = $entities;

        $userIds = array_filter(array_unique($posts->pluckNamed('user_id')));
        if (count($userIds) < 2) {
            return $controller->error(\XF::phrase(
                'sv_modess_please_select_posts_by_at_least_two_different_users'
            ));
        }

        $firstPostCount = 0;
        $total = $posts->count();
        $canHardDelete = true;

        foreach ($posts as $post) {
            /** @var \XF\Entity\Post $post */
            if ($post->post_id == $post->Thread->first_post_id) {
                $firstPostCount++;
            }

            if ($canHardDelete && !$post->canDelete('hard')) {
                $canHardDelete = false;
            }
        }

        $viewParams = [
            'posts'          => $posts,
            'firstPostCount' => $firstPostCount,
            'total'          => $total,
            'canHardDelete'  => $canHardDelete
        ];
        return $controller->view(
            'SV:Public:InlineMod\Post\ConvertToConversation',
            'sv_modess_inline_mod_post_convert_to_conversation',
            $viewParams
        );
    }

    /**
     * @param AbstractCollection $entities
     * @param \XF\Http\Request   $request
     *
     * @return array
     */
    public function getFormOptions(
        AbstractCollection $entities,
        \XF\Http\Request $request
    ) {
        $input = $request->filter([
            'hard_delete' => 'bool',
            'reason'      => 'str',
            'message'     => 'str'
        ]);

        return [
            'type'    => $input['hard_delete'] ? 'hard' : 'soft',
            'reason'  => $input['reason'],
            'message' => $input['message']
        ];
    }
}
