<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\Job\Upgrade;

use XF\Job\AbstractRebuildJob;

/**
 * Updates warning log entries to the core format.
 */
class WarningLogs extends AbstractRebuildJob
{
    /**
     * @param int $start
     * @param int $batch
     *
     * @return array
     */
    protected function getNextIds($start, $batch)
    {
        $db = $this->app->db();

        return $db->fetchAllColumn(
            $db->limit(
                "SELECT moderator_log_id
                    FROM xf_moderator_log
                    WHERE content_type = 'modess'
                        AND moderator_log_id > ?
                    ORDER BY moderator_log_id",
                $batch
            ),
            $start
        );
    }

    /**
     * @param int $id
     */
    protected function rebuildById($id)
    {
        /** @var \XF\Entity\ModeratorLog $entry */
        $entry = $this->app->em()->find('XF:ModeratorLog', $id);
        if (!$entry) {
            return;
        }

        if ($entry->action == 'warn') {
            // duplicates may be logged in core
            // the core records are less detailed, so remove them instead
            $this->app->db()->delete(
                'xf_moderator_log',
                "content_type = 'user'
                    AND action = 'warn'
                    AND content_id = ?
                    AND log_date = ?",
                [$entry->content_user_id, $entry->log_date]
            );
        }

        if ($entry->action == 'warn_update') {
            $expiryDate = isset($entry->action_params['expiry_date'])
                ? $entry->action_params['expiry_date']
                : 0;
            $logDate = $entry->log_date;

            if (!$expiryDate || ($expiryDate !== $logDate)) {
                // the warning expiry was changed, no core analogue
                $actionParams = $entry->action_params;
                $previousExpiryDate = isset($actionParams['previous_expiry_date'])
                    ? $actionParams['previous_expiry_date']
                    : 0;

                $entry->bulkSet([
                    'content_type'  => 'user',
                    'content_id'    => $entry->content_user_id,
                    'content_title' => $entry->content_username,
                    'action'        => 'warning_updated',
                    'action_params' => ['previous_expiry_date' => $previousExpiryDate]
                ]);
                $entry->save();
                return;
            }
        }

        $actionMap = [
            'warn'        => 'warning_given',
            'warn_update' => 'warning_expired',
            'warn_delete' => 'warning_removed'
        ];

        $entry->bulkSet([
            'content_type'  => $entry->discussion_content_type,
            'content_id'    => $entry->discussion_content_id,
            'content_title' => $entry->content_username,

            'discussion_content_type' => 'user',
            'discussion_content_id'   => $entry->content_user_id,

            'action'        => $actionMap[$entry->action],
            'action_params' => []
        ]);
        $entry->save();
    }

    /**
     * @return \XF\Phrase
     */
    protected function getStatusType()
    {
        return \XF::phrase('moderator_log_entry');
    }

    /**
     * @return bool
     */
    public function canCancel()
    {
        return false;
    }

    /**
     * @return bool
     */
    public function canTriggerByChoice()
    {
        return false;
    }
}
