<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\Job;

use XF\Job\AbstractJob;

/**
 * Reverts a user's message edits within a given time period.
 */
class UserRevertMessageEdit extends AbstractJob
{
    /**
     * @var array
     */
    protected $defaultData = [
        'userId' => null,
        'after'  => null,
        'before' => null,
        'start'  => 0,
        'batch'  => 100
    ];

    /**
     * @param int $maxRunTime
     *
     * @return \XF\Job\JobResult
     */
    public function run($maxRunTime)
    {
        $startTime = microtime(true);

        if (
            !$this->data['userId']
            || ($this->data['after'] === null)
            || ($this->data['before'] === null)
        ) {
            return $this->complete();
        }

        /** @var \XF\Mvc\Entity\Finder $historyFinder */
        $historyFinder = $this->app->finder('XF:EditHistory')
            ->where('edit_user_id', $this->data['userId'])
            ->where('edit_date', '>', $this->data['after'])
            ->where('edit_date', '<', $this->data['before'])
            ->where('edit_history_id', '>', $this->data['start'])
            ->order('edit_history_id')
            ->limit($this->data['batch']);

        /** @var \XF\Mvc\Entity\AbstractCollection $edits */
        $edits = $historyFinder->fetch();
        if (!$edits->count()) {
            return $this->complete();
        }

        /** @var \XF\Repository\EditHistory $historyRepo */
        $historyRepo = $this->app->repository('XF:EditHistory');

        $done = 0;
        foreach ($edits as $edit) {
            /** @var \XF\Entity\EditHistory $edit */
            $this->data['start'] = $edit->edit_history_id;

            $historyRepo->revertToHistory($edit);

            $done++;

            if ($maxRunTime && ((microtime(true) - $startTime) > $maxRunTime)) {
                break;
            }
        }

        $this->data['batch'] = $this->calculateOptimalBatch(
            $this->data['batch'],
            $done,
            $startTime,
            $maxRunTime,
            1000
        );
        return $this->resume();
    }

    /**
     * @return string
     */
    public function getStatusMessage()
    {
        return sprintf(
            '%s... (%s)',
            \XF::phrase('reverting_edits'),
            \XF::language()->numberFormat($this->data['start'])
        );
    }

    /**
     * @return bool
     */
    public function canCancel()
    {
        return true;
    }

    /**
     * @return bool
     */
    public function canTriggerByChoice()
    {
        return false;
    }
}
