<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\Pub\Controller;

use SV\ModeratorEssentials\XF\Entity\ModeratorLog;

use XF\Entity\Forum;
use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

/**
 * A control panel for moderators.
 */
class Moderator extends AbstractController
{
    /**
     * @param string       $action
     * @param ParameterBag $params
     */
    protected function preDispatchController($action, ParameterBag $params)
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canViewModeratorPanel()) {
            throw $this->exception($this->noPermission());
        }
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionIndex()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if ($visitor->canViewModeratorLog()) {
            return $this->redirect($this->buildLink('moderator/log'));
        }
        if ($visitor->hasPermission('general', 'banUser')) {
            return $this->redirect($this->buildLink('moderator/ban-user'));
        }
        if ($visitor->canViewBannedUserList()) {
            return $this->redirect($this->buildLink('moderator/banned-users'));
        }
        if ($visitor->canSearchIps()) {
            return $this->redirect($this->buildLink('moderator/ip-search'));
        }
        if ($visitor->canManageTags()) {
            return $this->redirect($this->buildLink('moderator/tags'));
        }
        if ($visitor->canManageReplyBans()) {
            return $this->redirect($this->buildLink('moderator/thread-reply-bans'));
        }

        // nothing to see
        return $this->noPermission();
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionLog()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canViewModeratorLog($error)) {
            return $this->noPermission($error);
        }

        $logRepo = $this->getModeratorLogRepo();
        $invalidSearch = $this->filter('invalid_search', 'bool');

        $searchId = $this->filter('search_id', 'uint');
        if (!$searchId) {
            $filters = $this->getLogFilterInput();
            if ($filters) {
                $session = false;
                if (!isset($filters['days'])) {
                    $filters['days'] = 1;
                }
                $logFinder = $logRepo->findLogsForList()->limit(200);
            } else {
                $session = true;
                $filters = $logRepo->getUserDefaultFilters($visitor->user_id);
                $logFinder = $logRepo->findSessionLogsForList();
            }

            $search = $logRepo->runSearch($logFinder, $filters);

            if ($session) {
                $this->session()->svModLogLastRead = \XF::$time;
                $logRepo->updateSessionLogCounts(0);
            }

            return $this->redirect($this->buildLink('moderator/log', null, [
                'search_id'      => $search->search_id,
                'invalid_search' => $invalidSearch
            ]));
        }

        /** @var \SV\ModeratorEssentials\Entity\Search $search */
        $search = $this->em()->find('SV\ModeratorEssentials:Search', $searchId);
        if (!$search) {
            return $this->redirect($this->buildLink('moderator/log', null, [
                'invalid_search' => true
            ]));
        }

        $total = $search->result_count;
        $page = $this->filterPage();
        $perPage = $this->options()->modess_mod_log_per_page;
        $offset = (($page - 1) * $perPage);

        $logIds = array_slice(
            $search->search_results,
            $offset,
            $perPage
        );
        if ($logIds) {
            $logFinder = $logRepo->findLogsForList()
                ->with('Discussion')
                ->where('moderator_log_id', $logIds);

            if ($visitor->canViewModeratorLogDiscussions()) {
                $logFinder->with([
                    'Discussion',
                    'Discussion.Forum.Node.Permissions|'.$visitor->permission_combination_id
                ]);
            }

            /** @var \XF\Mvc\Entity\AbstractCollection $logs */
            $logs = $logFinder->fetch();
            $logRepo->addContentToLogs($logs);
            $logs = $logs->filterViewable();
        } else {
            $logs = $this->em()->getEmptyCollection();
        }

        $more = (($offset + $perPage) < $total);
        if ($logs->count() && !$more) {
            $before = $logs->last()->log_date;
        } else {
            $before = null;
        }

        $filters = $search->search_params;
        $type = isset($filters['type']) ? $filters['type'] : null;
        $available = $logRepo->getAvailableFilters($type);

        $viewParams = [
            'search'        => $search,
            'logs'          => $logs,
            'invalidSearch' => $invalidSearch,
            'before'        => $before,

            'filters'   => $filters,
            'available' => $available,

            'total'   => $total,
            'page'    => $page,
            'perPage' => $perPage
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Moderators\Log',
            'sv_modess_moderators_log',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'log');
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionLogFilters()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canViewModeratorLog($error)) {
            return $this->noPermission($error);
        }

        $filters = $this->getLogFilterInput();
        $logRepo = $this->getModeratorLogRepo();

        if ($this->isPost()) {
            if ($this->filter('save', 'bool')) {
                $logRepo->saveUserDefaultFilters(
                    \XF::visitor()->user_id,
                    $filters
                );
            }

            return $this->redirect($this->buildLink(
                'moderator/log',
                null,
                $filters
            ));
        }

        $type = isset($filters['type']) ? $filters['type'] : null;
        $available = $logRepo->getAvailableFilters($type);
        if (empty($filters['actions'])) {
            $filters['actions'] = array_keys($available['actions']);
        }

        $viewParams = [
            'available' => $available,
            'filters'   => $filters
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Moderators\LogFilters',
            'sv_modess_moderators_log_filters',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'log');
    }

    /**
     * @return array
     */
    protected function getLogFilterInput()
    {
        $filters = [];

        $input = $this->filter([
            'type'    => 'str',
            'actions' => 'array-str',
            'users'   => 'str',
            'days'    => 'uint',
            'before'  => 'uint'
        ]);

        $type = $input['type'] ?: null;
        $available = $this->getModeratorLogRepo()->getAvailableFilters($type);

        if ($input['type']) {
            if (isset($available['type'][$input['type']])) {
                $filters['type'] = $input['type'];
            }
        }

        if (
            $input['actions']
            && array_diff(array_keys($available['actions']), $input['actions'])
        ) {
            $filters['actions'] = array_intersect(
                $input['actions'],
                array_keys($available['actions'])
            );
        }

        if ($input['users'] == 'exclude_self') {
            $filters['users'] = $input['users'];
        } else {
            $input['users'] = $this->filter('users', 'uint');
            if (isset($available['users'][$input['users']])) {
                $filters['users'] = $input['users'];
            }
        }

        if ($input['days']) {
            if (isset($available['days'][$input['days']])) {
                $filters['days'] = $input['days'];
            }
        }

        if ($input['before']) {
            $filters['before'] = $input['before'];
        }

        return $filters;
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionLogDiscuss()
    {
        $logId = $this->filter('log_id', 'uint');
        $log = $this->assertViewableModeratorLog($logId, ['Discussion']);

        if ($log->Discussion) {
            if (!$log->hasViewableDiscussion($error)) {
                return $this->noPermission($error);
            }

            $thread = $log->Discussion;
        } else {
            if (!$log->canStartDiscussion($error)) {
                return $this->noPermission($error);
            }

            $forum = $this->getModeratorLogRepo()->getDiscussionForum();
            if (!$forum) {
                return $this->error(\XF::phrase('requested_forum_not_found'));
            }

            $creator = $this->setupModeratorLogDiscussionCreation($forum, $log);
            if (!$creator->validate($errors)) {
                return $this->error($errors);
            }

            /** @var \XF\Entity\Thread $thread */
            $thread = $creator->save();
            $log->fastUpdate('modess_discussion_id', $thread->thread_id);
            $this->finalizeModeratorLogDiscussionCreation($creator);
        }

        return $this->redirect($this->buildLink('threads', $thread));
    }

    /**
     * @param Forum                                          $forum
     * @param \SV\ModeratorEssentials\XF\Entity\ModeratorLog $log
     *
     * @return \XF\Service\Thread\Creator
     */
    protected function setupModeratorLogDiscussionCreation(
        Forum $forum,
        ModeratorLog $log
    ) {
        /** @var \XF\Service\Thread\Creator $creator */
        $creator = $this->service('XF:Thread\Creator', $forum);
        $creator->setIsAutomated();

        $creator->setContent(
            $log->getDiscussionTitle(),
            $log->getDiscussionMessage(),
            false
        );

        $thread = $creator->getThread();
        $thread->bulkSet([
            'discussion_type'  => 'modess_action_discussion',
            'discussion_state' => 'visible'
        ]);

        return $creator;
    }

    /**
     * @param \XF\Service\Thread\Creator $creator
     */
    protected function finalizeModeratorLogDiscussionCreation(
        \XF\Service\Thread\Creator $creator
    ) {
        $thread = $creator->getThread();

        /** @var \XF\Repository\Thread $threadRepo */
        $threadRepo = $this->repository('XF:Thread');
        $threadRepo->markThreadReadByVisitor($thread);

        /** @var \XF\Repository\ThreadWatch $threadWatchRepo */
        $threadWatchRepo = $this->repository('XF:ThreadWatch');
        $threadWatchRepo->autoWatchThread($thread, \XF::visitor(), true);

        $creator->sendNotifications();
    }

    /**
     * @param int   $logId
     * @param array $extraWith
     *
     * @return \SV\ModeratorEssentials\XF\Entity\ModeratorLog
     */
    protected function assertViewableModeratorLog($logId, array $extraWith = [])
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\ModeratorLog $log */
        $log = $this->em()->find('XF:ModeratorLog', $logId, $extraWith);
        if (!$log) {
            throw $this->exception($this->notFound(\XF::phrase(
                'sv_modess_log_discussion_not_found'
            )));
        }

        if (!$log->canView($error)) {
            throw $this->exception($this->noPermission($error));
        }

        return $log;
    }

    /**
     * @return \SV\ModeratorEssentials\XF\Repository\ModeratorLog|\XF\Mvc\Entity\Repository
     */
    protected function getModeratorLogRepo()
    {
        return $this->repository('XF:ModeratorLog');
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionBanUser()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->hasPermission('general', 'banUser')) {
            return $this->noPermission();
        }

        $username = $this->filter('username', 'str');
        if ($username) {
            /** @var \XF\Entity\User $user */
            $user = $this->em()->findOne('XF:User', ['username' => $username]);
            if (!$user) {
                return $this->message(
                    \XF::phrase('specified_member_cannot_be_found_enter_entire_name')
                );
            }

            $params = new ParameterBag(['user_id' => $user->user_id]);
            return $this->rerouteController('XF:Member', 'ban', $params);
        }

        $viewParams = [];
        $view = $this->view(
            'SV\ModeratorEssentials:Moderators\BanUser',
            'sv_modess_moderators_ban_user',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'ban-user');
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionBannedUsers()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canViewBannedUserList($error)) {
            return $this->noPermission($error);
        }

        $page = $this->filterPage();
        $perPage = $this->options()->membersPerPage;

        /** @var \XF\Repository\Banning $banningRepo */
        $banningRepo = $this->repository('XF:Banning');
        /** @var \XF\Mvc\Entity\Finder $banFinder */
        $banFinder = $banningRepo->findUserBansForList()
            ->limitByPage($page, $perPage);

        $total = $banFinder->total();
        $this->assertValidPage(
            $page,
            $perPage,
            $total,
            'moderator/banned-users'
        );

        /** @var \XF\Mvc\Entity\AbstractCollection $users */
        $userBans = $banFinder->fetch();

        $viewParams = [
            'userBans' => $userBans,

            'page'    => $page,
            'perPage' => $perPage,
            'total'   => $total
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Moderators\BannedUsers',
            'sv_modess_moderators_banned_users',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'banned-users');
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionIpSearch()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canSearchIps($error)) {
            return $this->noPermission($error);
        }

        if ($this->isPost()) {
            $ip = $this->filter('ip', 'str');
            $parsed = \XF\Util\Ip::parseIpRangeString($ip);
            if (!$parsed) {
                return $this->message(
                    \XF::phrase('please_enter_valid_ip_or_ip_range')
                );
            }

            /** @var \XF\Repository\Ip $ipRepo */
            $ipRepo = $this->repository('XF:Ip');
            if ($parsed['isRange']) {
                $ips = $ipRepo->getUsersByIpRange(
                    $parsed['startRange'],
                    $parsed['endRange']
                );
            } else {
                $ips = $ipRepo->getUsersByIp($parsed['startRange']);
            }

            if (!$ips) {
                return $this->message(\XF::phrase('no_users_logged_at_ip'));
            }

            $viewParams = [
                'ip'          => $ip,
                'ipParsed'    => $parsed,
                'ipPrintable' => $parsed['printable'],
                'ips'         => $ips
            ];
            $view = $this->view(
                'SV\ModeratorEssentials:Moderators\IpList',
                'sv_modess_moderators_ip_list',
                $viewParams
            );
            return $this->addModeratorsWrapperParams($view, 'ip-search');
        }

        $viewParams = [];
        $view = $this->view(
            'SV\ModeratorEssentials:Moderators\IpSearch',
            'sv_modess_moderators_ip_search',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'ip-search');
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionThreadReplyBans()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canManageReplyBans($error)) {
            return $this->noPermission($error);
        }

        /** @var \XF\Repository\ThreadReplyBan $replyBanRepo */
        $replyBanRepo = $this->repository('XF:ThreadReplyBan');
        $replyBanFinder = $replyBanRepo->findReplyBansForList();
        $linkParams = [];

        $user = null;
        if ($username = $this->filter('username', 'str')) {
            $user = $this->finder('XF:User')
                ->where('username', $username)
                ->fetchOne();
            if ($user) {
                $replyBanFinder->where('user_id', $user->user_id);
                $linkParams['username'] = $user->username;
            }
        }

        $page = $this->filterPage();
        $perPage = 25;
        $total = $replyBanFinder->total();

        $this->assertValidPage(
            $page,
            $perPage,
            $total,
            'moderator/thread-reply-bans'
        );

        $bans = $replyBanFinder->limitByPage($page, $perPage)->fetch();

        $viewParams = [
            'bans' => $bans,
            'user' => $user,

            'page'    => $page,
            'perPage' => $perPage,
            'total'   => $total,

            'linkParams' => $linkParams
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Moderators\ThreadReplyBans',
            'sv_modess_thread_reply_ban_list',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'thread-reply-bans');
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionThreadReplyBanDelete()
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canManageReplyBans($error)) {
            return $this->noPermission($error);
        }

        $banId = $this->filter('ban_id', 'uint');
        $replyBan = $this->assertThreadReplyBanExists($banId);

        if (!$replyBan->Thread->canReplyBan($error)) {
            $this->noPermission($error);
        }

        if ($this->isPost()) {
            $replyBan->delete();
            return $this->redirect($this->buildLink(
                'moderator/thread-reply-bans'
            ));
        }

        $viewParams = [
            'ban' => $replyBan
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Moderators\ThreadReplyBanDelete',
            'sv_modess_thread_reply_ban_delete',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'thread-reply-bans');
    }

    /**
     * @param string            $id
     * @param array|string|null $with
     * @param string|null       $phraseKey
     *
     * @return \XF\Entity\ThreadReplyBan
     */
    protected function assertThreadReplyBanExists(
        $id,
        array $with = [],
        $phraseKey = null
    ) {
        $with[] = 'Thread';
        $with[] = 'User';

        return $this->assertRecordExists(
            'XF:ThreadReplyBan',
            $id,
            $with,
            $phraseKey
        );
    }

    /**
     * @param \XF\Mvc\Reply\View $view
     * @param string             $selected
     *
     * @return \XF\Mvc\Reply\View
     */
    protected function addModeratorsWrapperParams(
        \XF\Mvc\Reply\View $view,
        $selected
    ) {
        $view->setParam('pageSelected', $selected);
        return $view;
    }

    /**
     * @param array $activities
     *
     * @return string
     */
    public static function getActivityDetails(array $activities)
    {
        return \XF::phrase('performing_moderation_duties');
    }
}
