<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\Pub\Controller;

use XF\Entity\Tag as TagEntity;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

/**
 * A controller for managing tags.
 */
class Tag extends AbstractController
{
    /**
     * @param string       $action
     * @param ParameterBag $params
     */
    protected function preDispatchController($action, ParameterBag $params)
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canViewModeratorPanel()) {
            throw $this->exception($this->noPermission());
        }

        if (!$visitor->canManageTags()) {
            throw $this->exception($this->noPermission());
        }
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionIndex(ParameterBag $params)
    {
        $tagFinder = $this->finder('XF:Tag');
        $this->applyTagListFilters($tagFinder, $filters);

        $page = $this->filterPage();
        $perPage = 50;
        $total = $tagFinder->total();
        $this->assertValidPage($page, $perPage, $total, 'moderator/tags');

        $tags = $tagFinder->limitByPage($page, $perPage)->fetch();

        $viewParams = [
            'tags'    => $tags,
            'filters' => $filters,

            'page'    => $page,
            'perPage' => $perPage,
            'total'   => $total
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Tag\Listing',
            'sv_modess_tag_list',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'tags');
    }

    /**
     * @param \XF\Mvc\Entity\Finder $tagFinder
     * @param array                 $filters
     */
    protected function applyTagListFilters(
        \XF\Mvc\Entity\Finder $tagFinder,
        &$filters
    ) {
        $filters = [];

        $containing = $this->filter('containing', 'str');
        if ($containing !== '') {
            $tagFinder->where(
                'tag',
                'LIKE',
                $tagFinder->escapeLike($containing, '%?%')
            );
            $filters['containing'] = $containing;
        }

        $order = $this->filter('order', 'str');
        switch ($order) {
            case 'use_count':
            case 'last_use_date':
                $tagFinder->order($order, 'DESC');
                $filters['order'] = $order;
                break;

            default:
                $tagFinder->order('tag');
                break;
        }
    }

    /**
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionAdd()
    {
        $tag = $this->em()->create('XF:Tag');
        $tag->permanent = 1;
        return $this->tagAddEdit($tag);
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionEdit(ParameterBag $params)
    {
        $tag = $this->assertTagExists($params->tag_id);
        return $this->tagAddEdit($tag);
    }

    /**
     * @param TagEntity $tag
     *
     * @return \XF\Mvc\Reply\View
     */
    protected function tagAddEdit(TagEntity $tag)
    {
        $viewParams = [
            'tag' => $tag
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Tag/Edit',
            'sv_modess_tag_edit',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'tags');
    }

    /**
     * @param TagEntity $tag
     *
     * @return FormAction
     */
    protected function tagSaveProcess(TagEntity $tag)
    {
        $form = $this->formAction();

        $tag->setOption('admin_edit', true);

        $input = $this->filter([
            'tag'       => 'str',
            'tag_url'   => 'str',
            'permanent' => 'bool'
        ]);
        $form->basicEntitySave($tag, $input);

        return $form;
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionSave(ParameterBag $params)
    {
        $this->assertPostOnly();

        if ($params->tag_id) {
            $tag = $this->assertTagExists($params->tag_id);
        } else {
            $tag = $this->em()->create('XF:Tag');
        }

        $this->tagSaveProcess($tag)->run();
        return $this->redirect($this->buildLink('moderator/tags'));
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionDelete(ParameterBag $params)
    {
        $tag = $this->assertTagExists($params->tag_id);
        if (!$tag->preDelete()) {
            return $this->error($tag->getErrors());
        }

        if ($this->isPost()) {
            $tag->delete();
            return $this->redirect($this->buildLink('moderator/tags'));
        }

        $viewParams = [
            'tag' => $tag
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Tag\Delete',
            'sv_modess_tag_delete',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'tags');
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionMerge(ParameterBag $params)
    {
        $tag = $this->assertTagExists($params->tag_id);
        if (!$tag->preDelete()) {
            return $this->error($tag->getErrors());
        }

        if ($this->isPost()) {
            $sourceTag = $tag;

            $targetTagName = $this->filter('target', 'str');
            $targetTag = $this->finder('XF:Tag')
                ->where('tag', $targetTagName)
                ->fetchOne();
            if (!$targetTag) {
                return $this->error(\XF::phrase('requested_tag_not_found'));
            }

            if ($sourceTag->tag_id == $targetTag->tag_id) {
                return $this->error(
                    \XF::phrase('you_may_not_merge_tag_with_itself')
                );
            }

            /** @var \XF\Service\Tag\Merger $merger */
            $merger = $this->service('XF:Tag\Merger', $targetTag);
            $merger->merge($sourceTag);

            return $this->redirect($this->buildLink('moderator/tags'));
        }

        $viewParams = [
            'tag' => $tag
        ];
        $view = $this->view(
            'SV\ModeratorEssentials:Tag\Merge',
            'sv_modess_tag_merge',
            $viewParams
        );
        return $this->addModeratorsWrapperParams($view, 'tags');
    }

    /**
     * @param string            $id
     * @param array|string|null $with
     * @param string|null       $phraseKey
     *
     * @return TagEntity
     */
    protected function assertTagExists($id, $with = null, $phraseKey = null)
    {
        return $this->assertRecordExists('XF:Tag', $id, $with, $phraseKey);
    }

    /**
     * @return \XF\Repository\Tag
     */
    protected function getTagRepo()
    {
        return $this->repository('XF:Tag');
    }

    /**
     * @param \XF\Mvc\Reply\View $view
     * @param string             $selected
     *
     * @return \XF\Mvc\Reply\View
     */
    protected function addModeratorsWrapperParams(
        \XF\Mvc\Reply\View $view,
        $selected
    ) {
        $view->setParam('pageSelected', $selected);
        return $view;
    }

    /**
     * @param array $activities
     *
     * @return string
     */
    public static function getActivityDetails(array $activities)
    {
        return \XF::phrase('performing_moderation_duties');
    }
}
