<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\Service\Post;

use XF\Entity\ConversationMessage;
use XF\Entity\Post;
use XF\Mvc\Entity\AbstractCollection;
use XF\Service\AbstractService;

/**
 * A service for converting posts into a conversation.
 */
class ConversationConverter extends AbstractService
{
    /**
     * @var AbstractCollection
     */
    protected $posts;

    /**
     * @var array
     */
    protected $options;

    /**
     * @var array
     */
    protected $defaultOptions = [
        'type'    => 'soft',
        'reason'  => '',
        'message' => ''
    ];

    /**
     * @var \XF\Entity\ConversationMaster
     */
    protected $conversation;

    /**
     * @var array
     */
    protected $conversationMessages;

    /**
     * @var array
     */
    protected $messageManagers;

    /**
     * @var array
     */
    protected $recipients;

    /**
     * @var \XF\Repository\Conversation
     */
    protected $conversationRepo;

    /**
     * @param \XF\App            $app
     * @param AbstractCollection $posts
     * @param array              $options
     */
    public function __construct(
        \XF\App $app,
        AbstractCollection $posts,
        array $options = []
    ) {
        parent::__construct($app);

        $this->setPosts($posts);
        $this->setOptions($options);

        $this->conversation = $this->em()->create('XF:ConversationMaster');

        $this->conversationMessages = [];
        $this->messageManagers = [];
        $this->recipients = [];

        $this->conversationRepo = $this->repository('XF:Conversation');
    }

    /**
     * @return AbstractCollection $posts
     */
    protected function setPosts(AbstractCollection $posts)
    {
        $posts = \XF\Util\Arr::columnSort($posts->toArray(), 'post_date');

        foreach ($posts as $post) {
            /** @var \XF\Entity\Post $post */
            $post->setOption('log_moderator', false);
        }

        $this->posts = $this->em()->getBasicCollection($posts);
    }

    /**
     * @return AbstractCollection
     */
    public function getPosts()
    {
        return $this->posts;
    }

    /**
     * @param array $options
     */
    public function setOptions(array $options)
    {
        $this->options = array_merge($this->defaultOptions, $options);
    }

    /**
     * @return array
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @return \XF\Entity\ConversationMaster
     */
    public function getConversation()
    {
        return $this->conversation;
    }

    /**
     * @return AbstractCollection
     */
    public function getConversationMessages()
    {
        return $this->conversationMessages;
    }

    /**
     * @return array
     */
    public function getRecipients()
    {
        return $this->recipients;
    }

    /**
     * @return \XF\Entity\ConversationMaster
     */
    public function convert()
    {
        $posts = $this->posts;
        $options = $this->options;
        $conversation = $this->conversation;

        $this->setupConversation($posts->first());
        foreach ($posts as $postId => $post) {
            $this->createPostConversationMessage($post);
        }
        if ($options['message']) {
            $this->createModeratorConversationMessage($options['message']);
        }
        $this->finalizeConversation();

        $db = $this->db();
        $db->beginTransaction();

        $conversation->save(true, false);
        $this->afterConversationInsert();

        $this->deletePosts($options['type'], $options['reason']);

        foreach ($posts as $post) {
            $this->app->logger()->logModeratorAction(
                'post',
                $post,
                'convert_to_conversation',
                ['reason' => $options['reason']]
            );
        }

        $db->commit();
        return $conversation;
    }

    /**
     * @param Post $post
     */
    protected function setupConversation(Post $post)
    {
        $conversation = $this->conversation;

        $conversation->title = \XF::phrase(
            'sv_modess_posts_moved_from_x',
            ['title' => $post->Thread->title]
        );
        $conversation->user_id = $post->user_id;
        $conversation->username = $post->username;
        $conversation->start_date = $post->post_date;
    }

    /**
     * @param Post $post
     */
    protected function createPostConversationMessage(Post $post)
    {
        $conversation = $this->conversation;

        /** @var \XF\Entity\ConversationMessage $message */
        $message = $conversation->getNewMessage($post->User);
        /** @var \XF\Service\Conversation\MessageManager $messageManager */
        $messageManager = $this->service(
            'XF:Conversation\MessageManager',
            $message
        );
        $messageManager->setLogIp(false);

        $conversation->addCascadedSave($message);
        $message->hydrateRelation('Conversation', $conversation);

        $message->message_date = $post->post_date;
        $messageManager->setMessage($post->message, true, false);

        $this->conversationMessages[$post->post_id] = $message;
        $this->messageManagers[$post->post_id] = $messageManager;
        $this->recipients[$post->user_id] = $post->User;
    }

    /**
     * @param string $messageContent
     */
    protected function createModeratorConversationMessage($messageContent)
    {
        $visitor = \XF::visitor();
        $conversation = $this->conversation;

        /** @var \XF\Entity\ConversationMessage $message */
        $message = $conversation->getNewMessage($visitor);
        /** @var \XF\Service\Conversation\MessageManager $messageManager */
        $messageManager = $this->service(
            'XF:Conversation\MessageManager',
            $message
        );
        $messageManager->setLogIp(false);

        $conversation->addCascadedSave($message);
        $message->hydrateRelation('Conversation', $conversation);

        $message->message_date = \XF::$time;
        $messageManager->setMessage($messageContent, true, false);

        $this->conversationMessages[0] = $message;
        $this->messageManagers[0] = $messageManager;
        $this->recipients[$visitor->user_id] = $visitor;
    }

    /**
     * Prepares the conversation for insert.
     */
    protected function finalizeConversation()
    {
        $conversation = $this->conversation;
        $conversationMessages = $this->conversationMessages;

        /** @var \XF\Entity\ConversationMessage $lastMessage */
        $lastMessage = end($conversationMessages);

        $conversation->last_message_date = $lastMessage->message_date;
        $conversation->last_message_user_id = $lastMessage->user_id;
        $conversation->last_message_username = $lastMessage->username;
        $conversation->reply_count = (count($conversationMessages) - 1);
    }

    /**
     * Conversation post-insert actions.
     */
    protected function afterConversationInsert()
    {
        $posts = $this->posts;
        $conversation = $this->conversation;
        $conversationMessages = $this->conversationMessages;
        $recipients = $this->recipients;
        $conversationRepo = $this->conversationRepo;

        foreach ($conversationMessages as $postId => $message) {
            /** @var \XF\Entity\ConversationMessage $message */
            /** @var \XF\Service\Conversation\MessageManager $messageManager */
            $messageManager = $this->messageManagers[$postId];
            $messageManager->afterInsert();

            /** @var \XF\Entity\Post $post */
            $post = isset($posts[$postId]) ? $posts[$postId] : null;
            if ($post) {
                $message->attach_count = $post->attach_count;
                $message->embed_metadata = $this->updateEmbeds(
                    $post,
                    $message,
                    $post->embed_metadata ?: []
                );
                $message->saveIfChanged();
            }
        }

        /** @var \XF\Entity\ConversationMessage $lastMessage */
        $firstMessage = reset($conversationMessages);
        /** @var \XF\Entity\ConversationMessage $lastMessage */
        $lastMessage = end($conversationMessages);

        $conversation->fastUpdate([
            'first_message_id' => $firstMessage->message_id,
            'last_message_id'  => $lastMessage->message_id
        ]);

        $conversationRepo->insertRecipients(
            $conversation,
            $recipients
        );

        $visitor = \XF::visitor();
        if (isset($recipients[$visitor->user_id])) {
            $userConv = $conversation->Users[$visitor->user_id];
            $conversationRepo->markUserConversationRead($userConv);

            unset($recipients[$visitor->user_id]);
        }

        /** @var \XF\Service\Conversation\Notifier $notifier */
        $notifier = $this->service('XF:Conversation\Notifier', $conversation);
        $notifier->addNotificationLimit($recipients)->notifyCreate();
    }

    /**
     * @param Post                $post
     * @param ConversationMessage $message
     * @param array               $embedMetadata
     *
     * @return array
     */
    protected function updateEmbeds(
        Post $post,
        ConversationMessage $message,
        array $embedMetadata
    ) {
        $attachEmbed = isset($embedMetadata['attachments'])
            ? $embedMetadata['attachments']
            : [];

        foreach ($post->Attachments as $sourceAttachment) {
            /** @var \XF\Entity\Attachment $sourceAttachment */
            /** @var \XF\Entity\Attachment $targetAttachment */
            $targetAttachment = $this->em()->create('XF:Attachment');

            $values = $sourceAttachment->toArray(false);
            unset(
                $values['attachment_id'],
                $values['content_type'],
                $values['content_id'],
                $values['view_count']
            );

            $targetAttachment->content_type = 'conversation_message';
            $targetAttachment->content_id = $message->message_id;
            $targetAttachment->bulkSet($values);
            $targetAttachment->save();

            $message->message = preg_replace(
                '#(\[attach(=[^]]+)?\])'.$sourceAttachment->attachment_id.'(\[/attach\])#i',
                '${1}'.$targetAttachment->attachment_id.'${3}',
                $message->message
            );

            if (isset($attachEmbed[$sourceAttachment->attachment_id])) {
                unset($attachEmbed[$sourceAttachment->attachment_id]);
                $attachEmbed[$targetAttachment->attachment_id] = $targetAttachment->attachment_id;
            }
        }

        if ($attachEmbed) {
            $embedMetadata['attachments'] = $attachEmbed;
        } else {
            unset($embedMetadata['attachments']);
        }

        return $embedMetadata;
    }

    /**
     * @param string $type
     * @param string $reason
     */
    protected function deletePosts($type, $reason = '')
    {
        foreach ($this->posts as $post) {
            /** @var \XF\Service\Post\Deleter $deleter */
            $deleter = $this->service('XF:Post\Deleter', $post);
            $deleter->delete($type, $reason);
        }
    }
}
