<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\Service\ThreadNotes;

use SV\ModeratorEssentials\Entity\ThreadNotes;

use XF\Entity\Forum;
use XF\Entity\Thread;
use XF\Entity\User;
use XF\Service\AbstractService;
use XF\Service\ValidateAndSavableTrait;

/**
 * A service for creating thread notes.
 */
class Creator extends AbstractService
{
    use ValidateAndSavableTrait;

    /**
     * @var ThreadNotes
     */
    protected $notes;

    /**
     * @var Thread
     */
    protected $thread;

    /**
     * @var Forum
     */
    protected $forum;

    /**
     * @var User
     */
    protected $user;

    /**
     * @var \XF\Service\Thread\Creator
     */
    protected $threadCreator;

    /**
     * @param \XF\App $app
     * @param Thread  $thread
     * @param Forum   $forum
     * @param User    $user
     */
    public function __construct(
        \XF\App $app,
        Thread $thread,
        Forum $forum,
        User $user
    ) {
        parent::__construct($app);

        $this->setupDefaults();
        $this->setThread($thread);
        $this->setForum($forum);
        $this->setUser($user);
    }

    /**
     * Sets up defaults.
     */
    protected function setupDefaults()
    {
        $this->notes = $this->em()->create('SV\ModeratorEssentials:ThreadNotes');
    }

    /**
     * @return ThreadNotes
     */
    public function getNotes()
    {
        return $this->notes;
    }

    /**
     * @param Thread $thread
     */
    protected function setThread(Thread $thread)
    {
        $this->thread = $thread;

        $this->notes->thread_id = $this->thread->thread_id;
        $this->notes->content_user_id = $this->thread->user_id;
        $this->notes->content_username = $this->thread->username;
    }

    /**
     * @return Thread
     */
    public function getThread()
    {
        return $this->thread;
    }

    /**
     * @param Forum $forum
     */
    protected function setForum(Forum $forum)
    {
        $this->forum = $forum;
        $this->notes->forum_id = $this->forum->node_id;
    }

    /**
     * @return Forum
     */
    public function getForum()
    {
        return $this->forum;
    }

    /**
     * @param User $user
     */
    protected function setUser(User $user)
    {
        $this->user = $user;
        $this->notes->user_id = $user->user_id;
    }

    /**
     * @return User
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * Finalizes setup.
     */
    protected function finalSetup()
    {
        $this->threadCreator = $this->setupNotesThreadCreation($this->forum);

        /** @var \XF\Entity\Thread $thread */
        $thread = $this->threadCreator->getThread();
        $this->notes->discussion_thread = $this->em()->getDeferredValue(
            function () use ($thread) {
                return $thread->thread_id;
            },
            'save'
        );
    }

    /**
     * @return array
     */
    protected function _validate()
    {
        $this->finalSetup();

        $this->notes->preSave();
        $errors = $this->notes->getErrors();

        if (!$this->threadCreator->validate($threadErrors)) {
            $errors = array_merge($errors, $threadErrors);
        }

        return $errors;
    }

    /**
     * Saves the notes entity.
     */
    protected function _save()
    {
        $notes = $this->notes;

        $db = $this->db();
        $db->beginTransaction();

        /** @var Thread $notesThread */
        $notesThread = $this->threadCreator->save();
        $this->afterNotesThreadCreated($notesThread);
        $notes->save(true, false);

        $db->commit();

        return $notes;
    }

    /**
     * @param Forum $forum
     *
     * @return \XF\Service\Thread\Creator
     */
    protected function setupNotesThreadCreation(Forum $forum)
    {
        /** @var \XF\Service\Thread\Creator $creator */
        $creator = $this->service('XF:Thread\Creator', $forum);
        $creator->setIsAutomated();

        $creator->setContent(
            $this->getNotesThreadTitle(),
            $this->getNotesThreadMessage(),
            false
        );

        $thread = $creator->getThread();
        $thread->bulkSet([
            'discussion_type'  => 'sv_thread_notes',
            'discussion_state' => 'visible'
        ]);

        return $creator;
    }

    /**
     * @param Thread $thread
     */
    protected function afterNotesThreadCreated(Thread $thread)
    {
        /** @var \XF\Repository\Thread $threadRepo */
        $threadRepo = $this->repository('XF:Thread');
        $threadRepo->markThreadReadByVisitor($thread);

        /** @var \XF\Repository\ThreadWatch $threadWatchRepo */
        $threadWatchRepo = $this->repository('XF:ThreadWatch');
        $threadWatchRepo->autoWatchThread($thread, \XF::visitor(), true);
    }

    /**
     * @return string
     */
    protected function getNotesThreadTitle()
    {
        $message = \XF::phrase('sv_modess_thread_notes_title', [
            'thread_title' => $this->thread->title
        ]);
        return $message->render('raw');
    }

    /**
     * @return string
     */
    protected function getNotesThreadMessage()
    {
        $thread = $this->thread;

        $router = $this->app->router('public');
        $link = $router->buildLink('full:threads', $thread);

        $message = \XF::phrase('sv_modess_thread_notes_message', [
            'username'        => $this->user->username,
            'thread_link'     => $link,
            'thread_title'    => $thread->title,
            'thread_username' => $thread->User ? $thread->User->username : $thread->username
        ]);
        return $message->render('raw');
    }

    /**
     * Sends user notifications.
     */
    public function sendNotifications()
    {
        $this->threadCreator->sendNotifications();
    }
}
