<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

/**
 * Handles installation, upgrades, and uninstallation of the add-on.
 */
class Setup extends AbstractSetup
{
    // from https://github.com/Xon/XenForo2-Utils cloned to src/addons/SV/Utils
    use \SV\Utils\InstallerHelper;
    use StepRunnerInstallTrait;
    use StepRunnerUninstallTrait;
    use StepRunnerUpgradeTrait;

    /**
     * Creates add-on tables.
     */
    public function installStep1()
    {
        $sm = $this->schemaManager();

        foreach ($this->getTables() as $tableName => $callback)
        {
            $sm->createTable($tableName, $callback);
            $sm->alterTable($tableName, $callback);
        }
    }

    /**
     * Alters core tables.
     */
    public function installStep2()
    {
        $sm = $this->schemaManager();

        foreach ($this->getAlterTables() as $tableName => $callback)
        {
            $sm->alterTable($tableName, $callback);
        }
    }

    /**
     * Creates add-on tables.
     */
    public function upgrade2000000Step1()
    {
        $this->installStep1();
    }

    /**
     * Alters core tables.
     */
    public function upgrade2000000Step2()
    {
        $this->installStep2();
    }

    /**
     * Alters add-on tables.
     */
    public function upgrade2000000Step3()
    {
        $sm = $this->schemaManager();

        foreach ($this->getTables() as $tableName => $callback)
        {
            $sm->alterTable($tableName, $callback);
        }
    }

    /**
     * Prunes old add-on data.
     */
    public function upgrade2000000Step4()
    {
        \XF::registry()->delete('modLogCounts');

        // options haven't been created yet. Create a deferred task todo it after this
        \XF::app()->jobManager()->enqueueUnique(
            'SV\ModeratorEssentials',
            'SV\ModeratorEssentials:InstallHelper',
            [],
            false
        );
    }

    /**
     * Prunes old add-on data.
     */
    public function upgrade2000013Step1()
    {
        $this->db()->delete(
            'xf_content_type_field',
            'content_type = ?',
            'modess'
        );
    }

    public function upgrade2010000Step1()
    {
        $this->applyGlobalPermission('general', 'modess_access', 'general', 'modessThreadViewModLogs');
        $this->applyGlobalPermission('general', 'modess_access', 'general', 'canViewModLogCounts');
        $this->applyGlobalPermission('general', 'modess_access', 'general', 'modess_IpSearch');

        $this->applyGlobalPermission('general', 'modess_manageReplyBans', 'general', 'modessThreadViewModLogs');
        $this->applyGlobalPermission('general', 'modess_manageReplyBans', 'general', 'canViewModLogCounts');
        $this->applyGlobalPermission('general', 'modess_manageReplyBans', 'general', 'modess_IpSearch');
    }

    /**
     * @param int   $previousVersion
     * @param array $stateChanges
     */
    public function postUpgrade($previousVersion, array &$stateChanges)
    {
        if ($previousVersion && ($previousVersion < 2000012)) {
            $this->app->jobManager()->enqueueUnique(
                'svModessLegacyThreadNotes',
                'SV\ModeratorEssentials:Upgrade\ThreadNotes'
            );
        }

        if ($previousVersion && ($previousVersion < 2000013)) {
            $this->app->jobManager()->enqueueUnique(
                'svModessLegacyWarningLogs',
                'SV\ModeratorEssentials:Upgrade\WarningLogs'
            );
        }
    }

    /**
     * Drops add-on tables.
     */
    public function uninstallStep1()
    {
        $sm = $this->schemaManager();

        foreach ($this->getTables() as $tableName => $callback)
        {
            $sm->dropTable($tableName);
        }
    }

    /**
     * Drops columns from core tables.
     */
    public function uninstallStep2()
    {
        $sm = $this->schemaManager();

        foreach ($this->getRemoveAlterTables() as $tableName => $callback)
        {
            $sm->alterTable($tableName, $callback);
        }
    }

    /**
     * Removes add-on data from the registry.
     */
    public function uninstallStep3()
    {
        \XF::registry()->delete('svModEssModLogLM');
    }

    /**
     * @return array
     */
    protected function getTables()
    {
        $tables = [];

        $tables['xf_moderator_discussion'] = function ($table) {
            /** @var Create|Alter $table */
            $this->addOrChangeColumn($table, 'discussion_id', 'int')->autoIncrement()->primaryKey();
            $this->addOrChangeColumn($table, 'thread_id', 'int');
            $this->addOrChangeColumn($table, 'forum_id', 'int');
            $this->addOrChangeColumn($table, 'discussion_thread', 'int');
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'content_user_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'content_username', 'varchar', 100)->setDefault('');
            $table->addKey('discussion_thread');
            $table->addKey('thread_id');
        };

        $tables['modess_modlog_preferences'] = function ($table) {
            /** @var Create|Alter $table */
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'action_type', 'varchar', 25)->setDefault('');
            $this->addOrChangeColumn($table, 'action_names', 'blob')->nullable(true)->setDefault(null);
            $table->addPrimaryKey('user_id');
        };

        $tables['modess_modlog_search'] = function ($table) {
            /** @var Create|Alter $table */
            $this->addOrChangeColumn($table, 'search_id', 'int')->autoIncrement()->primaryKey();
            $this->addOrChangeColumn($table, 'search_results', 'mediumblob');
            $this->addOrChangeColumn($table, 'result_count', 'smallint');
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'search_date', 'int');
            $this->addOrChangeColumn($table, 'search_params', 'blob');
        };

        return $tables;
    }

    /**
     * @return array
     */
    protected function getAlterTables()
    {
        $tables = [];

        $tables['xf_moderator_log'] = function (Alter $table) {
            $this->addOrChangeColumn($table, 'modess_discussion_id', 'int')->setDefault(0);
            $table->addKey('modess_discussion_id');
        };

        return $tables;
    }

    protected function getRemoveAlterTables()
    {
        $tables = [];

        $tables['xf_moderator_log'] = function (Alter $table) {
            $table->dropColumns(['modess_discussion_id']);
            $table->dropIndexes(['modess_discussion_id']);
        };

        return $tables;
    }

    /**
     * @param int|null $previousVersion
     *
     * @return bool True if permissions were applied.
     */
    protected function applyDefaultPermissions($previousVersion = null)
    {
        // TODO: default permissions?
        $applied = false;

        if (!$previousVersion) {

            $applied = true;
        }

        if (!$previousVersion || $previousVersion <= 2070900) {

            $applied = true;
        }

        return $applied;
    }
}
