<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\XF\Entity;

use SV\ModeratorEssentials\ModeratorLog\PanelInterface;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * Extends \XF\Entity\ModeratorLog
 *
 * @property int|null modess_discussion_id
 * @property Entity Content
 * @property Thread Discussion
 */
class ModeratorLog extends XFCP_ModeratorLog
{
    /**
     * @param string $error
     *
     * @return bool
     */
    public function canView(&$error = null)
    {
        $handler = $this->getHandler();

        if (!$handler) {
            return false;
        }

        if (!($handler instanceof PanelInterface)) {
            return false;
        }

        if (!$handler->canViewLogs()) {
            return false;
        }

        return $handler->canViewLog($this, $error);
    }

    /**
     * @param string $error
     *
     * @return bool
     */
    public function canStartDiscussion(&$error = null)
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canViewModeratorLogDiscussions($error)) {
            return false;
        }

        return $this->modess_discussion_id ? false : true;
    }

    /**
     * @param string $error
     *
     * @return bool
     */
    public function hasViewableDiscussion(&$error = null)
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canViewModeratorLogDiscussions($error)) {
            return false;
        }

        if (!$this->modess_discussion_id) {
            return false;
        }
        if (!$this->Discussion) {
            return false;
        }

        return $this->Discussion->canView($error);
    }

    /**
     * @return \XF\ModeratorLog\AbstractHandler
     */
    public function getHandler()
    {
        return $this->getModeratorLogRepo()->getModeratorLogHandler(
            $this->content_type
        );
    }

    /**
     * @param Entity $content
     */
    public function setContent(Entity $content = null)
    {
        $this->_getterCache['Content'] = $content;
    }

    /**
     * @return Entity
     */
    public function getContent()
    {
        $handler = $this->getHandler();
        return $handler ? $handler->getContent($this->content_id) : null;
    }

    /**
     * @return string
     */
    public function getDiscussionTitle()
    {
        return $this->action_text;
    }

    /**
     * @return string
     */
    public function getDiscussionMessage()
    {
        $pather = $this->app()->container('request.pather');
        $message = \XF::phrase('sv_modess_moderator_log_discussion_message', [
            'username'         => \XF::visitor()->username,
            'content_link'     => $pather($this->content_url, 'full'),
            'content_title'    => $this->content_title_,
            'content_username' => $this->content_username,
            'log_action'       => $this->action_text,
            'log_username'     => $this->User->username,
            'log_date'         => \XF::language(0)->date($this->log_date)
        ]);
        return $message->render('raw');
    }

    /**
     * Entity post-save events.
     */
    public function _postSave()
    {
        parent::_postSave();

        $this->updateLastModifiedDate();
    }

    /**
     * Entity post-delete events.
     */
    public function _postDelete()
    {
        parent::_postDelete();

        if (
            $this->modess_discussion_id
            && $this->Discussion
            && ($this->Discussion->discussion_type == 'modess_action_discussion')
        ) {
            $thread = $this->Discussion;
            $thread->discussion_open = false;
            $thread->saveIfChanged($saved, false, false);
        }

        $this->updateLastModifiedDate();
    }

    /**
     * Updates the last modified date for the moderator log.
     */
    protected function updateLastModifiedDate()
    {
        $handler = $this->getHandler();
        if ($handler && ($handler instanceof PanelInterface)) {
            \XF::runOnce('svModLogLastModified', function () {
                $this->getModeratorLogRepo()->updateLastModifiedDate();
            });
        }
    }

    /**
     * @param Structure $structure
     *
     * @return Structure
     */
    public static function getStructure(Structure $structure)
    {
        $structure = parent::getStructure($structure);
        $visitor = \XF::visitor();

        $structure->columns['modess_discussion_id'] = [
            'type'    => self::UINT,
            'default' => 0
        ];
        $structure->getters['Content'] = true;
        $structure->relations['Discussion'] = [
            'entity'     => 'XF:Thread',
            'type'       => self::TO_ONE,
            'conditions' => [['thread_id', '=', '$modess_discussion_id']],
            'primary'    => true,
            'with'       => [
                'Forum',
                'Forum.Node',
                'Forum.Node.Permissions|'.$visitor->permission_combination_id
            ]
        ];

        return $structure;
    }

    /**
     * @return \SV\ModeratorEssentials\XF\Repository\ModeratorLog|\XF\Mvc\Entity\Repository
     */
    protected function getModeratorLogRepo()
    {
        return $this->repository('XF:ModeratorLog');
    }
}
