<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\XF\Pub\Controller;

use XF\Mvc\ParameterBag;

/**
 * Extends \XF\Pub\Controller\Member
 */
class Member extends XFCP_Member
{
    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionBanSave(ParameterBag $params)
    {
        $reply = parent::actionBanSave($params);

        if ($reply instanceof \XF\Mvc\Reply\Redirect) {
            $referrer = $this->app()->request()->getReferrer();
            if ($referrer == $this->buildLink('full:moderators/ban-user')) {
                return $this->redirect($this->buildLink(
                    'moderator/banned-users'
                ));
            }

            /** @var \XF\Mvc\Reply\Redirect $reply */
            return $this->redirect($this->getDynamicRedirect($reply->getUrl()));
        }

        return $reply;
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionBanLift(ParameterBag $params)
    {
        $reply = parent::actionBanLift($params);

        if ($reply instanceof \XF\Mvc\Reply\Redirect) {
            $referrer = $this->app()->request()->getReferrer();
            if ($referrer == $this->buildLink('full:moderators/ban-user')) {
                return $this->redirect($this->buildLink(
                    'moderator/banned-users'
                ));
            }

            /** @var \XF\Mvc\Reply\Redirect $reply */
            return $this->redirect($this->getDynamicRedirect($reply->getUrl()));
        }

        return $reply;
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionRevertMessageEdit(ParameterBag $params)
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $user */
        $user = $this->assertViewableUser($params->user_id);

        if (!$user->canRevertMessageEdits($error)) {
            return $this->noPermission($error);
        }

        $options = $this->options();

        $warningId = $options->modess_rollback_warning;
        /** @var \XF\Entity\WarningDefinition $warningDefinition */
        $warningDefinition = $this->em()->find(
            'XF:WarningDefinition',
            $warningId
        );

        if ($this->isPost()) {
            $after = $this->filter('after', 'array');
            $after = $this->filterArray($after, [
                'date'   => 'datetime',
                'hour'   => 'uint',
                'minute' => 'uint'
            ]);
            $before = $this->filter('before', 'array');
            $before = $this->filterArray($before, [
                'date'   => 'datetime',
                'hour'   => 'uint',
                'minute' => 'uint'
            ]);
            $after = ($after['date'] + ($after['hour'] * 3600) + ($after['minute'] * 60));
            $before = ($before['date'] + ($before['hour'] * 3600) + ($before['minute'] * 60));

            $editHistoryOptions = $options->editHistory;
            if ($editHistoryOptions['length']) {
                $after = max(
                    $after,
                    (\XF::$time - ($editHistoryOptions['length'] * 86400))
                );
            }
            $before = min($before, \XF::$time);

            if ($after >= $before) {
                return $this->error(\XF::phrase(
                    'sv_modess_please_enter_a_valid_time_period'
                ));
            }

            if ($this->filter('warn', 'bool') && $warningDefinition) {
                /** @var \XF\Service\User\Warn $warner */
                $warner = $this->service(
                    'XF:User\Warn',
                    $user,
                    'user',
                    $user,
                    \XF::visitor()
                );

                $points = $this->filter('apply_points', 'bool') ? null : 0;
                $warner->setFromDefinition($warningDefinition, $points);

                $warner->setNotes(\XF::phrase('sv_modess_revert_warning_note'));

                list($conversationTitle, $conversationMessage) = $warningDefinition->getSpecificConversationContent(
                    $user,
                    'user',
                    $user
                );
                if (strlen($conversationTitle) && strlen($conversationMessage)) {
                    $options = [
                        'open_invite'       => false,
                        'conversation_open' => true
                    ];
                    $warner->withConversation(
                        $conversationTitle,
                        $conversationMessage,
                        $options
                    );
                }

                if (!$warner->validate($errors)) {
                    return $this->error($errors);
                }
                $warner->save();
            }

            $this->app()->jobManager()->enqueue(
                'SV\ModeratorEssentials:UserRevertMessageEdit',
                [
                    'userId' => $user->user_id,
                    'after'  => $after,
                    'before' => $before
                ],
                true
            );
            return $this->redirect($this->buildLink('members', $user));
        }

        $time = [];
        $time['hours'] = [];
        for ($i = 0; $i < 24; $i++) {
            $hh = str_pad($i, 2, '0', STR_PAD_LEFT);
            $time['hours'][$hh] = $hh;
        }
        $time['minutes'] = [];
        for ($i = 0; $i < 60; $i++) {
            $mm = str_pad($i, 2, '0', STR_PAD_LEFT);
            $time['minutes'][$mm] = $mm;
        }

        $viewParams = [
            'user'              => $user,
            'warningDefinition' => $warningDefinition,
            'time'              => $time
        ];
        return $this->view(
            'XF:Member\RevertMessageEdit',
            'sv_modess_member_revert_message_edit',
            $viewParams
        );
    }

    /**
     * @param ParameterBag $params
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    public function actionDeletedPosts(ParameterBag $params)
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\User $user */
        $user = $this->assertViewableUser($params->user_id);

        if (!$user->canViewAllDeletedPosts($error)) {
            return $this->noPermission($error);
        }

        $page = $this->filterPage($params->page);
        $perPage = $this->app()->options()->messagesPerPage;
        $this->assertCanonicalUrl(
            $this->buildLink('members/deleted-posts', $user, ['page' => $page])
        );

        /** @var \XF\Finder\Post $postFinder */
        $postFinder = $this->finder('XF:Post')
            ->with(['Thread', 'Thread.Forum', 'DeletionLog'])
            ->where('user_id', $user->user_id)
            ->where('message_state', 'deleted')
            ->order([
                ['DeletionLog.delete_date', 'desc'],
                ['post_date', 'desc']
            ])
            ->limitByPage($page, $perPage);

        $cache = $this->app()->cache();
        if ($cache) {
            $cacheId = "sv_modess_deleted_posts_{$user->user_id}";

            if ($cache->contains($cacheId)) {
                $total = $cache->fetch($cacheId);
            } else {
                $total = $postFinder->total();
                $cache->save($cacheId, $total, 900);
            }
        } else {
            $total = $postFinder->total();
        }
        $this->assertValidPage($page, $perPage, $total, 'members', $user);

        $posts = $postFinder->fetch();

        $canInlineMod = false;
        foreach ($posts as $post) {
            /** @var \XF\Entity\Post $post */
            if ($post->canUseInlineModeration()) {
                $canInlineMod = true;
                break;
            }
        }

        $viewParams = [
            'user'         => $user,
            'posts'        => $posts,
            'canInlineMod' => $canInlineMod,

            'page'    => $page,
            'perPage' => $perPage,
            'total'   => $total
        ];
        return $this->view(
            'XF:Member\DeletedPosts',
            'sv_modess_member_deleted_posts',
            $viewParams
        );
    }
}
