<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\XF\Repository;

use SV\ModeratorEssentials\Entity\DefaultFilters;
use SV\ModeratorEssentials\ModeratorLog\PanelInterface;

use XF\Entity\ModeratorLog as Log;
use XF\Entity\Thread;

/**
 * Extends \XF\Repository\ModeratorLog
 */
class ModeratorLog extends XFCP_ModeratorLog
{
    /**
     * @return \XF\Mvc\Entity\Finder
     */
    public function findSessionLogsForList()
    {
        $session = $this->app()->session();
        $lastReadDate = $session->mvModLogLastRead ?: $session->previousActivity;
        $max = $this->options()->modess_mod_log_max_entries;

        return $this->findLogsForList()
            ->where('log_date', '>', $lastReadDate)
            ->limit($max);
    }

    /**
     * @param Thread $thread
     *
     * @return \XF\Mvc\Entity\Finder
     */
    public function findLogForLogDiscussion(Thread $thread)
    {
        /** @var \XF\Mvc\Entity\Finder $finder */
        $finder = $this->finder('XF:ModeratorLog')
            ->with('User')
            ->where('modess_discussion_id', $thread->thread_id);

        return $finder;
    }

    /**
     * @param \XF\Mvc\Entity\Finder $logFinder
     * @param array                 $filters
     *
     * @return \SV\ModeratorEssentials\Entity\Search|\XF\Mvc\Entity\Entity
     */
    public function runSearch(\XF\Mvc\Entity\Finder $logFinder, array $filters)
    {
        // TODO: reuse cached searches within time frame? (add filter_hash w/ index) (log)
        $this->applyFilters($logFinder, $filters);

        /** @var \XF\Mvc\Entity\AbstractCollection $logs */
        $logs = $logFinder->fetch();
        $this->addContentToLogs($logs);
        $logs = $logs->filterViewable();

        $search = $this->em->create('SV\ModeratorEssentials:Search');
        $search->bulkSet([
            'search_results' => $logs->pluckNamed('moderator_log_id'),
            'user_id'        => \XF::visitor()->user_id,
            'search_params'  => $filters
        ]);
        $search->save();

        return $search;
    }

    /**
     * @param \XF\Mvc\Entity\Finder $logFinder
     * @param array                 $filters
     */
    public function applyFilters(
        \XF\Mvc\Entity\Finder $logFinder,
        array $filters
    ) {
        $available = $this->getAvailableFilters();
        if (!isset($filters['type'])) {
            $filters['type'] = array_filter(array_keys($available['type']));
        }
        if (!isset($filters['actions'])) {
            $filters['actions'] = array_filter(array_keys($available['actions']));
        }

        if (!empty($filters['type'])) {
            $logFinder->where('content_type', $filters['type']);
        }

        if (!empty($filters['actions'])) {
            $logFinder->where('action', $filters['actions']);
        }

        if (!empty($filters['users'])) {
            if ($filters['users'] == 'exclude_self') {
                $logFinder->where('user_id', '!=', \XF::visitor()->user_id);
            } else {
                $logFinder->where('user_id', $filters['users']);
            }
        }

        if (!empty($filters['days'])) {
            $cutoff = (\XF::$time - ($filters['days'] * 86400));
            $logFinder->where('log_date', '>=', $cutoff);
        }

        if (!empty($filters['before'])) {
            $logFinder->where('log_date', '<', $filters['before']);
        }
    }

    /**
     * @var array
     */
    protected $availableFilters;

    /**
     * @param string $type
     *
     * @return array
     */
    public function getAvailableFilters($type = null)
    {
        if ($this->availableFilters === null) {
            $types = [
                '' => \XF::phrase('all')
            ];
            $handlers = $this->getModeratorLogHandlers();
            foreach ($handlers as $contentType => $handler) {
                if (($handler instanceof PanelInterface) && $handler->canViewLogs()) {
                    $types[$contentType] = $this->app()->getContentTypePhrase(
                        $contentType,
                        true
                    );
                }
            }

            $actions = $this->getLogActions();

            $users = [
                ''             => \XF::phrase('all'),
                'exclude_self' => \XF::phrase('sv_modess_all_excluding_self')
            ];
            /** @var \XF\Repository\Moderator $modRepo */
            $modRepo = $this->repository('XF:Moderator');
            $moderators = $modRepo->findModeratorsForList()->fetch();
            $users += $moderators->pluckNamed('User', 'user_id')->pluckNamed('username', 'user_id');

            $dayRange = range(1, $this->options()->modess_mod_log_search_max_days);
            $days = array_combine($dayRange, $dayRange);

            $this->availableFilters = [
                'type'    => $types,
                'actions' => $actions,
                'users'   => $users,
                'days'    => $days
            ];
        }

        $filters = $this->availableFilters;
        if ($type && isset($filters['type'][$type])) {
            $filters['actions'] = $filters['actions'][$type];
        } else {
            $filters['actions'] = array_merge(...array_values($filters['actions']));
        }
        return $filters;
    }

    /**
     * @return array
     */
    public function getLogActions()
    {
        $actions = [];

        $handlers = $this->getModeratorLogHandlers();
        foreach ($handlers as $contentType => $handler) {
            if (!($handler instanceof PanelInterface)) {
                continue;
            }

            /** @var PanelInterface $handler */
            $actions[$contentType] = $this->addPhrasesToLogActions(
                $handler->getActions()
            );
        }

        if (!$actions) {
            return [];
        }

        return $actions;
    }

    /**
     * @param array $actions
     *
     * @return array
     */
    public function addPhrasesToLogActions(array $actions)
    {
        $phrased = [];
        foreach ($actions as $action) {
            $phrased[$action] = \XF::phrase("sv_modess_actions.{$action}");
        }
        return $phrased;
    }

    /**
     * @param int $userId
     *
     * @return array
     */
    public function getUserDefaultFilters($userId)
    {
        $filters = ['users' => 'exclude_self'];

        /** @var DefaultFilters $default */
        $default = $this->em->find(
            'SV\ModeratorEssentials:DefaultFilters',
            $userId
        );
        if ($default) {
            $filters['type'] = $default->action_type;
            $filters['actions'] = $default->action_names;
        }

        return $filters;
    }

    /**
     * @param int   $userId
     * @param array $filters
     *
     * @return bool
     */
    public function saveUserDefaultFilters($userId, array $filters)
    {
        if (!$userId) {
            return null;
        }

        /** @var DefaultFilters $default */
        $default = $this->em->find(
            'SV\ModeratorEssentials:DefaultFilters',
            $userId
        );
        if ($default && !isset($filters['type']) && !isset($filters['actions'])) {
            $default->delete();
            return false;
        }

        if (!$default) {
            $default = $this->em->create(
                'SV\ModeratorEssentials:DefaultFilters'
            );
            $default->user_id = $userId;
        }

        $default->action_type = isset($filters['type']) ? $filters['type'] : '';
        $default->action_names = isset($filters['actions']) ? $filters['actions'] : [];
        $default->save();

        return true;
    }

    /**
     * @param \XF\Mvc\Entity\AbstractCollection $logs
     * @param bool                              $requiredOnly
     */
    public function addContentToLogs(
        \XF\Mvc\Entity\AbstractCollection $logs,
        $requiredOnly = true
    ) {
        $contentMap = [];
        foreach ($logs as $logId => $log) {
            /** @var Log $log */
            $contentType = $log->content_type;
            if (!isset($contentMap[$contentType])) {
                $contentMap[$contentType] = [];
            }
            $contentMap[$contentType][$logId] = $log->content_id;
        }

        $handlers = $this->getModeratorLogHandlers();
        foreach ($contentMap as $contentType => $contentIds) {
            if (!isset($handlers[$contentType])) {
                continue;
            }

            /** @var \XF\ModeratorLog\AbstractHandler $handler */
            $handler = $handlers[$contentType];
            if ($requiredOnly) {
                if (!($handler instanceof PanelInterface)) {
                    continue;
                }
                if (!$handler->requiresHydration()) {
                    continue;
                }
            }

            $data = $handler->getContent($contentIds);
            foreach ($contentIds as $logId => $contentId) {
                $content = isset($data[$contentId]) ? $data[$contentId] : null;
                $logs[$logId]->setContent($content);
            }
        }
    }

    /**
     * @return int
     */
    public function updateLastModifiedDate()
    {
        $lastModified = time();
        \XF::registry()->set('svModEssModLogLM', $lastModified);
        return $lastModified;
    }

    /**
     * @param int $count
     *
     * @return array
     */
    public function updateSessionLogCounts($count)
    {
        $logCounts = [
            'total'        => $count,
            'lastModified' => \XF::$time
        ];
        $this->app()->session()->svModLogCounts = $logCounts;
        return $logCounts;
    }

    /**
     * @return array
     */
    public function rebuildSessionLogCounts()
    {
        $visitor = \XF::visitor();
        $filters = $this->getUserDefaultFilters($visitor->user_id);

        $logFinder = $this->findSessionLogsForList();
        $this->applyFilters($logFinder, $filters);

        /** @var \XF\Mvc\Entity\AbstractCollection $logs */
        $logs = $logFinder->fetch();
        $this->addContentToLogs($logs);
        $logs = $logs->filterViewable();

        return $this->updateSessionLogCounts($logs->count());
    }

    /**
     * @param int $cutOff
     *
     * @return int
     */
    public function pruneModeratorLogs($cutOff = null)
    {
        $rows = parent::pruneModeratorLogs($cutOff);
        $this->updateLastModifiedDate();
        return $rows;
    }

    /**
     * @param int $cutOff
     *
     * @return int
     */
    public function pruneSearches($cutOff = null)
    {
        if ($cutOff === null) {
            $cutOff = (\XF::$time - 86400);
        }

        return $this->db()->delete(
            'modess_modlog_search',
            'search_date < ?',
            $cutOff
        );
    }

    /**
     * @return null|\XF\Entity\Forum|\XF\Mvc\Entity\Entity
     */
    public function getDiscussionForum()
    {
        $forumId = $this->options()->modess_action_discussion_node;
        if (!$forumId) {
            return null;
        }

        return $this->em->find('XF:Forum', $forumId, 'Node');
    }

    /**
     * @return array
     */
    public function getModeratorLogHandlers()
    {
        // TODO: XF bug: https://xenforo.com/community/threads/147746/
        $handlers = [];

        $handlerClasses = \XF::app()->getContentTypeField(
            'moderator_log_handler_class'
        );
        foreach ($handlerClasses as $contentType => $handlerClass) {
            if (class_exists($handlerClass)) {
                $handlerClass = \XF::extendClass($handlerClass);
                $handlers[$contentType] = new $handlerClass($contentType);
            }
        }

        return $handlers;
    }

    /**
     * @param string $type
     * @param bool   $throw
     *
     * @return \XF\ModeratorLog\AbstractHandler|null
     */
    public function getModeratorLogHandler($type, $throw = false)
    {
        // TODO: XF bug: https://xenforo.com/community/threads/147746/
        $handlerClass = \XF::app()->getContentTypeFieldValue(
            $type,
            'moderator_log_handler_class'
        );
        if (!$handlerClass || !class_exists($handlerClass)) {
            if ($throw) {
                throw new \InvalidArgumentException(
                    "No moderator log handler for '{$type}'"
                );
            }

            return null;
        }

        $handlerClass = \XF::extendClass($handlerClass);
        return new $handlerClass($type);
    }
}
